==> cvsnotify-1.0.0.tar.gz: cvsnotify-1.0.0/cvsnotify-contentsend.pl <==
#!/usr/bin/env perl
####################################################################
# CVS Activity Notifying Tool
# $Id: cvsnotify-contentsend.pl,v 1.2 2004/03/18 18:26:19 pynoos Exp $
# Copyright (C) 2004 by Hojin Choi <pynoos@users.kldp.net>
# You can redistribute this software under GPL.
####################################################################

sub contentsend_preprocess()
{
	return 1;
}

sub _replace($)
{
	$cont = shift;
	foreach( 0 .. $#contentsend_replace )
	{
		$contentsend_replace[$_] =~ /(.{1})([^\1]*)\1([^\1]*)\1/;
		my( $p, $r ) = ($2, $3 );
		$cont =~ s/$p/$r/mg;
	}
	return $cont;
}

sub extract_info_from_file($)
{
	my( $tmp_diff ) = shift;

	print $log "ContentSend: open $tmp_diff\n";
	unless( -f $tmp_diff )
	{
		print $log "There is no such a file [$tmp_diff]\n";
		return;
	}

	open FILE, "$tmp_diff" or print $log "Oops\n";

	my( $length, $title, @content );
	$length = 0;
	while(<FILE>)
	{
		$length++;
		print $log "T:$_";
		my $c = _replace($_);
		print $log "T:$c";
		my %tplhash = ( LINE => $c );
		$content[$length] = replace_template( $tpl{'CONTENT_EQUAL'}, %tplhash);
		if( $length > 20 )
		{
			next;
		}
		elsif( /.*title.*:\s*(.*)\s*$/i )
		{
			print $log "Title found : [$1]\n";
			$title = $1;
		}
	}
	close( FILE );
	return ($title, @content);
}

sub _do_diff($$@)
{
	my( $tmp_diff1, $tmp_diff2, @content ) = @_;

	print $log "ContentSend: " . $conf{'diff'} . " -u $tmp_diff2 $tmp_diff1 2>>$debug_logfile\n";
	open FILE, $conf{'diff'} . " -u $tmp_diff2 $tmp_diff1 2>>$debug_logfile |" or goto rollup;

	my( $section_begin, $length );
	$section_begin = 0;
	$length = 0;
	while(<FILE>)
	{
		print $log $_;
		if( /^Index:.*/ )
		{
			next;
		}
		elsif( /^diff.*/ or /^--/ or /^\+\+/ )
		{
			next;
		}
		elsif( /^@@ \-(\d+),(\d+) \+(\d+),(\d+) @@/ ) 
		{
			my( $old_line,$old_length,$expected_length);
			( $old_line,$old_length,$section_begin,$expected_length) = ($1,$2,$3,$4);
			$length = 0;
			for( $section_begin .. $section_begin+$expected_length - 1)
			{
				$content[$_] = "";
			}
			next;
		}

		my( $type, $tpl, $c, $index );
		$type = substr($_,0,1);
		$c = substr($_,1);
		$c = _replace($c);
		if( $type eq "-" )
		{
			$tpl = $tpl{'CONTENT_DROPPED'};
			$index = $section_begin + $length;
		}
		elsif( $type eq "+" )
		{
			$tpl = $tpl{'CONTENT_ADDED'};
			$index = $section_begin + $length;
			$length++;
		}
		else
		{
			$tpl = $tpl{'CONTENT_EQUAL'};
			$index = $section_begin + $length;
			$length++;
		}
		$tpl =~ s/{LINE}/$c/g;
		$content[$index] .= $tpl;
	}
	close( FILE );
	return join '', @content;
}

sub get_module_notify_email($)
{
	my( $file ) = shift;
	my( $check, $email );

	foreach( @{$conf_array{'diff_notify'}} )
	{
		next unless /((.{1})([^\2]*)\2)\s+(.*)\s*$/;
		my( $p, $e ) = ($3, $4 );
		print $log "[[Found $p -> $e]]\n";
		if( "$module/$file" =~ /$p/ )
		{
			$email .= ", " if $e;
			$email = $e;
		}
	}
	return $email;
}

sub content_date($)
{
	my ($tpl_) = shift;
	my ($module_file,$module_rev_new,$module_rev_old) = ( $$tpl_{'FILE'}, $$tpl_{'REV_NEW'}, $$tpl_{'REV_OLD'} );
	my ($date_rev1, $date_rev2);

	print $log "ContentSend: fetch date from $module_file\n";

	print $log `pwd`;
	print $log $conf{'cvs'} . " -n log -r$module_rev_new $module_file 2>>$debug_logfile\n";
	open FILE, $conf{'cvs'} . " -n log -r$module_rev_new $module_file 2>>$debug_logfile |" or return;
	$$tpl_{'DATE_NEW'} = "";
	while(<FILE>)
	{
		print $log $_;
		next unless /^date: ([^;]+);/;
		$$tpl_{'DATE_NEW'} = $1;
		last;
	}
	close( FILE );

	print $log $conf{'cvs'} . " -n log -r$module_rev_old $module_file 2>>$debug_logfile\n";
	open FILE, $conf{'cvs'} . " -n log -r$module_rev_old $module_file 2>>$debug_logfile |" or return;
	$$tpl_{'DATE_OLD'} = "";
	while(<FILE>)
	{
		next unless /^date: ([^;]+);/;
		$$tpl_{'DATE_OLD'} = $1;
		last;
	}
	close( FILE );
}

sub do_diff_notify(@)
{
	my( @content );
	my( $tmp_diff1, $tmp_diff2 );
	my( %tplhash );
	my( $email );

	($email, $tplhash{'FILE'}, $tplhash{'REV_OLD'}, $tplhash{'REV_NEW'} ) = @_;

	$email =~ s/^(['"]{1})(.*)(\1)$/\2/;
	return unless $email;

	$tplhash{'MODULE'} = $module;
	$tplhash{'URL_MODULE'}  = $conf{'cvsurl'}."/".$tplhash{'MODULE'};
	$tplhash{'URL_MODULE'} .= "?" . $conf{'subroot'} if $conf{'subroot'};
	$tplhash{'URL_FILE'}    = $tplhash{'URL_MODULE'}."/".$tplhash{'FILE'};
	$tplhash{'FROM'} = "$whoami\@" . $conf{'maildomain'};
	$tplhash{'LOGMSG'} = $loginfo_logmsg;

	$tmp_diff1 = "/tmp/cvsnotify-diff-$whoami-$$." . $tplhash{'REV_NEW'};
	$tmp_diff2 = "/tmp/cvsnotify-diff-$whoami-$$." . $tplhash{'REV_OLD'};

	_system( $conf{'cvs'} . " checkout -p -r" . $tplhash{'REV_NEW'} . " \"$tplhash{'MODULE'}/$tplhash{'FILE'}\" 2>>$debug_logfile >$tmp_diff1");

	( $tplhash{'TITLE'},  @content ) = extract_info_from_file( $tmp_diff1 );

	$tplhash{'TO'} = $email;
	$tplhash{'TITLE'} = "$tplhash{'MODULE'}/$tplhash{'FILE'}" unless $tplhash{'TITLE'};

	_system( $conf{'cvs'} . " checkout -p -r" . $tplhash{'REV_OLD'} . " \"$tplhash{'MODULE'}/$tplhash{'FILE'}\" 2>>$debug_logfile >$tmp_diff2");
	if( $tplhash{'REV_OLD'} ne 'NONE' )
	{
		print "cvsnotify-contentsend: Diff-marked '" . $tplhash{'FILE'} . "' (" . $tplhash{'REV_OLD'} . "," . $tplhash{'REV_NEW'} . ") will be sent to $email\n";
		$tplhash{'CONTENT'} = _do_diff( $tmp_diff1, $tmp_diff2, @content );
	}
	else
	{
		print "cvsnotify-contentsend: New '". $tplhash{'FILE'} . "' will be sent to $email\n";
		$tplhash{'CONTENT'} = "@content";
	}

	content_date(\%tplhash);

	_sendmail( replace_template( $tpl{'CONTENT_SEND_MESSAGE'}, %tplhash) );

rollup:
	unlink( $tmp_diff1 );
	unlink( $tmp_diff2 );
	print $log "ContentSend: Unlink $tmp_diff2\n";
}

=cut COPYRIGHT of encode_base64
From MIME::Base64.pm
Copyright 1995-1999, 2001 Gisle Aas.
=cut

sub encode_base64 ($;$)
{
    my $res = "";
    my $eol = $_[1];
    $eol = "\n" unless defined $eol;
    pos($_[0]) = 0;                          # ensure start at the beginning

    $res = join '', map( pack('u',$_)=~ /^.(\S*)/, ($_[0]=~/(.{1,45})/gs));

    $res =~ tr|` -_|AA-Za-z0-9+/|;               # `# help emacs
    # fix padding at the end
    my $padding = (3 - length($_[0]) % 3) % 3;
    $res =~ s/.{$padding}$/'=' x $padding/e if $padding;
    # break encoded string into lines of no more than 76 characters each
    if (length $eol) {
	$res =~ s/(.{1,76})/$1$eol/g;
    }
    return $res;
}

=cut End of embeded perl code
=cut

sub do_attach_notify(@)
{
	my( $buf );
	my( $content );
	my( %tplhash );
	my( $email );

	($email, $tplhash{'FILE'}, $tplhash{'REV_OLD'}, $tplhash{'REV_NEW'} ) = @_;

	$email =~ s/^(['"]{1})(.*)(\1)$/\2/;
	return unless $email;

	$tplhash{'MODULE'} = $module;
	$tplhash{'URL_MODULE'}  = $conf{'cvsurl'}."/".$tplhash{'MODULE'};
	$tplhash{'URL_MODULE'} .= "?" . $conf{'subroot'} if $conf{'subroot'};
	$tplhash{'URL_FILE'}    = $tplhash{'URL_MODULE'}."/".$tplhash{'FILE'};
	$tplhash{'FROM'} = "$whoami\@" . $conf{'maildomain'};
	$tplhash{'TO'} = $email;
	$tplhash{'TITLE'} = "$tplhash{'MODULE'}/$tplhash{'FILE'}";
	$tplhash{'LOGMSG'} = $loginfo_logmsg;

	content_date(\%tplhash);

	open CONTENT, $conf{'cvs'} . " checkout -p -r" . $tplhash{'REV_NEW'} . " \"$tplhash{'MODULE'}/$tplhash{'FILE'}\" 2>>$debug_logfile |" or return;
	while (read(CONTENT, $buf, 60*57)) 
	{
		$content .= encode_base64($buf);
	}
	close( CONTENT );
	$tplhash{'CONTENT'} = $content;

	open SENDMAIL, "|" . $conf{'sendmail'} . " -t" or return;
	print SENDMAIL replace_template( $tpl{'CONTENT_ATTACH_SEND_MESSAGE'}, %tplhash );
	close( SENDMAIL );
	if( $? )
	{
		print $log "Mail didn't sent with ". $conf{'sendmail'} . "\n";
	}
	else
	{
		print $log "Mail sent with ". $conf{'sendmail'} . "\n";
	}

	#print $log $content;
}

sub contentsend_foreach($$$)
{
	my($file, $rev_old, $rev_new) = @_;
	my($email);
	$email = get_matched_value( $file, "attach_notify", "cvsnotify-contentsend" );
	if( $email )
	{
		do_attach_notify( $email, $file, $rev_old, $rev_new );
	}
	$email = get_matched_value( $file, "diff_notify", "cvsnotify-contentsend" );
	if( $email )
	{
		do_diff_notify( $email, $file, $rev_old, $rev_new );
	}
}

sub contentsend_postprocess()
{
	return 1;
}

# End of script.
1

==> cvsnotify-1.1.0.tar.gz: cvsnotify-1.1.0/cvsnotify-contentsend.pl <==
#!/usr/bin/env perl
####################################################################
# CVS Activity Notifying Tool
# $Id: cvsnotify-contentsend.pl,v 1.3 2004/05/29 04:36:19 pynoos Exp $
# Copyright (C) 2004 by Hojin Choi <pynoos@users.kldp.net>
# You can redistribute this software under GPL.
####################################################################

sub contentsend_preprocess()
{
	return 1;
}

sub _replace($)
{
	$cont = shift;
	foreach( 0 .. $#contentsend_replace )
	{
		$contentsend_replace[$_] =~ /(.{1})([^\1]*)\1([^\1]*)\1/;
		my( $p, $r ) = ($2, $3 );
		$cont =~ s/$p/$r/mg;
	}
	return $cont;
}

sub extract_info_from_file($)
{
	my( $tmp_diff ) = shift;

	log_debug( "ContentSend: open $tmp_diff\n");
	unless( -f $tmp_diff )
	{
		log_debug( "There is no such a file [$tmp_diff]\n");
		return;
	}

	open FILE, "$tmp_diff" or log_debug( "Oops\n");

	my( $length, $title, @content );
	$length = 0;
	while(<FILE>)
	{
		$length++;
		log_debug( "T:$_");
		my $c = _replace($_);
		log_debug( "T:$c");
		my %tplhash = ( LINE => $c );
		$content[$length] = replace_template( $tpl{'CONTENT_EQUAL'}, %tplhash);
		if( $length > 20 )
		{
			next;
		}
		elsif( /.*title.*:\s*(.*)\s*$/i )
		{
			log_debug( "Title found : [$1]\n");
			$title = $1;
		}
	}
	close( FILE );
	return ($title, @content);
}

sub _do_diff($$@)
{
	my( $tmp_diff1, $tmp_diff2, @content ) = @_;

	log_debug( "ContentSend: " . $conf{'diff'} . " -u $tmp_diff2 $tmp_diff1 2>>$debug_logfile\n");
	open FILE, $conf{'diff'} . " -u $tmp_diff2 $tmp_diff1 2>>$debug_logfile |" or goto rollup;

	my( $section_begin, $length );
	$section_begin = 0;
	$length = 0;
	while(<FILE>)
	{
		log_debug( $_);
		if( /^Index:.*/ )
		{
			next;
		}
		elsif( /^diff.*/ or /^--/ or /^\+\+/ )
		{
			next;
		}
		elsif( /^@@ \-(\d+),(\d+) \+(\d+),(\d+) @@/ ) 
		{
			my( $old_line,$old_length,$expected_length);
			( $old_line,$old_length,$section_begin,$expected_length) = ($1,$2,$3,$4);
			$length = 0;
			for( $section_begin .. $section_begin+$expected_length - 1)
			{
				$content[$_] = "";
			}
			next;
		}

		my( $type, $tpl, $c, $index );
		$type = substr($_,0,1);
		$c = substr($_,1);
		$c = _replace($c);
		if( $type eq "-" )
		{
			$tpl = $tpl{'CONTENT_DROPPED'};
			$index = $section_begin + $length;
		}
		elsif( $type eq "+" )
		{
			$tpl = $tpl{'CONTENT_ADDED'};
			$index = $section_begin + $length;
			$length++;
		}
		else
		{
			$tpl = $tpl{'CONTENT_EQUAL'};
			$index = $section_begin + $length;
			$length++;
		}
		$tpl =~ s/{LINE}/$c/g;
		$content[$index] .= $tpl;
	}
	close( FILE );
	return join '', @content;
}

sub get_module_notify_email($)
{
	my( $file ) = shift;
	my( $check, $email );

	foreach( @{$conf_array{'diff_notify'}} )
	{
		next unless /((.{1})([^\2]*)\2)\s+(.*)\s*$/;
		my( $p, $e ) = ($3, $4 );
		log_debug( "[[Found $p -> $e]]\n");
		if( "$module/$file" =~ /$p/ )
		{
			$email .= ", " if $e;
			$email = $e;
		}
	}
	return $email;
}

sub content_date($)
{
	my ($tpl_) = shift;
	my ($module_file,$module_rev_new,$module_rev_old) = ( $$tpl_{'FILE'}, $$tpl_{'REV_NEW'}, $$tpl_{'REV_OLD'} );
	my ($date_rev1, $date_rev2);

	log_debug( "ContentSend: fetch date from $module_file\n");

	log_debug( `pwd`);
	log_debug( $conf{'cvs'} . " -n log -r$module_rev_new $module_file 2>>$debug_logfile\n");
	open FILE, $conf{'cvs'} . " -n log -r$module_rev_new $module_file 2>>$debug_logfile |" or return;
	$$tpl_{'DATE_NEW'} = "";
	while(<FILE>)
	{
		log_debug( $_);
		next unless /^date: ([^;]+);/;
		$$tpl_{'DATE_NEW'} = $1;
		last;
	}
	close( FILE );

	log_debug( $conf{'cvs'} . " -n log -r$module_rev_old $module_file 2>>$debug_logfile\n");
	open FILE, $conf{'cvs'} . " -n log -r$module_rev_old $module_file 2>>$debug_logfile |" or return;
	$$tpl_{'DATE_OLD'} = "";
	while(<FILE>)
	{
		next unless /^date: ([^;]+);/;
		$$tpl_{'DATE_OLD'} = $1;
		last;
	}
	close( FILE );
}

sub do_diff_notify(@)
{
	my( @content );
	my( $tmp_diff1, $tmp_diff2 );
	my( %tplhash );
	my( $email );

	($email, $tplhash{'FILE'}, $tplhash{'REV_OLD'}, $tplhash{'REV_NEW'} ) = @_;

	$email =~ s/^(['"]{1})(.*)(\1)$/\2/;
	return unless $email;

	$tplhash{'MODULE'} = $module;
	$tplhash{'URL_MODULE'}  = $conf{'cvsurl'}."/".$tplhash{'MODULE'};
	$tplhash{'URL_MODULE'} .= "?" . $conf{'subroot'} if $conf{'subroot'};
	$tplhash{'URL_FILE'}    = $tplhash{'URL_MODULE'}."/".$tplhash{'FILE'};
	$tplhash{'FROM'} = "$whoami\@" . $conf{'maildomain'};
	$tplhash{'LOGMSG'} = $loginfo_logmsg;

	$tmp_diff1 = "/tmp/cvsnotify-diff-$whoami-$$." . $tplhash{'REV_NEW'};
	$tmp_diff2 = "/tmp/cvsnotify-diff-$whoami-$$." . $tplhash{'REV_OLD'};

	_system( $conf{'cvs'} . " checkout -p -r" . $tplhash{'REV_NEW'} . " \"$tplhash{'MODULE'}/$tplhash{'FILE'}\" 2>>$debug_logfile >$tmp_diff1");

	( $tplhash{'TITLE'},  @content ) = extract_info_from_file( $tmp_diff1 );

	$tplhash{'TO'} = $email;
	$tplhash{'TITLE'} = "$tplhash{'MODULE'}/$tplhash{'FILE'}" unless $tplhash{'TITLE'};

	_system( $conf{'cvs'} . " checkout -p -r" . $tplhash{'REV_OLD'} . " \"$tplhash{'MODULE'}/$tplhash{'FILE'}\" 2>>$debug_logfile >$tmp_diff2");
	if( $tplhash{'REV_OLD'} ne 'NONE' )
	{
		print "cvsnotify-contentsend: Diff-marked '" . $tplhash{'FILE'} . "' (" . $tplhash{'REV_OLD'} . "," . $tplhash{'REV_NEW'} . ") will be sent to $email\n";
		$tplhash{'CONTENT'} = _do_diff( $tmp_diff1, $tmp_diff2, @content );
	}
	else
	{
		print "cvsnotify-contentsend: New '". $tplhash{'FILE'} . "' will be sent to $email\n";
		$tplhash{'CONTENT'} = "@content";
	}

	content_date(\%tplhash);

	_sendmail( replace_template( $tpl{'CONTENT_SEND_MESSAGE'}, %tplhash) );

rollup:
	unlink( $tmp_diff1 );
	unlink( $tmp_diff2 );
	log_debug( "ContentSend: Unlink $tmp_diff2\n");
}

=cut COPYRIGHT of encode_base64
From MIME::Base64.pm
Copyright 1995-1999, 2001 Gisle Aas.
=cut

sub encode_base64 ($;$)
{
    my $res = "";
    my $eol = $_[1];
    $eol = "\n" unless defined $eol;
    pos($_[0]) = 0;                          # ensure start at the beginning

    $res = join '', map( pack('u',$_)=~ /^.(\S*)/, ($_[0]=~/(.{1,45})/gs));

    $res =~ tr|` -_|AA-Za-z0-9+/|;               # `# help emacs
    # fix padding at the end
    my $padding = (3 - length($_[0]) % 3) % 3;
    $res =~ s/.{$padding}$/'=' x $padding/e if $padding;
    # break encoded string into lines of no more than 76 characters each
    if (length $eol) {
	$res =~ s/(.{1,76})/$1$eol/g;
    }
    return $res;
}

=cut End of embeded perl code
=cut

sub do_attach_notify(@)
{
	my( $buf );
	my( $content );
	my( %tplhash );
	my( $email );

	($email, $tplhash{'FILE'}, $tplhash{'REV_OLD'}, $tplhash{'REV_NEW'} ) = @_;

	$email =~ s/^(['"]{1})(.*)(\1)$/\2/;
	return unless $email;

	$tplhash{'MODULE'} = $module;
	$tplhash{'URL_MODULE'}  = $conf{'cvsurl'}."/".$tplhash{'MODULE'};
	$tplhash{'URL_MODULE'} .= "?" . $conf{'subroot'} if $conf{'subroot'};
	$tplhash{'URL_FILE'}    = $tplhash{'URL_MODULE'}."/".$tplhash{'FILE'};
	$tplhash{'FROM'} = "$whoami\@" . $conf{'maildomain'};
	$tplhash{'TO'} = $email;
	$tplhash{'TITLE'} = "$tplhash{'MODULE'}/$tplhash{'FILE'}";
	$tplhash{'LOGMSG'} = $loginfo_logmsg;

	content_date(\%tplhash);

	open CONTENT, $conf{'cvs'} . " checkout -p -r" . $tplhash{'REV_NEW'} . " \"$tplhash{'MODULE'}/$tplhash{'FILE'}\" 2>>$debug_logfile |" or return;
	while (read(CONTENT, $buf, 60*57)) 
	{
		$content .= encode_base64($buf);
	}
	close( CONTENT );
	$tplhash{'CONTENT'} = $content;

	open SENDMAIL, "|" . $conf{'sendmail'} . " -t" or return;
	print SENDMAIL replace_template( $tpl{'CONTENT_ATTACH_SEND_MESSAGE'}, %tplhash );
	close( SENDMAIL );
	if( $? )
	{
		log_debug( "Mail didn't sent with ". $conf{'sendmail'} . "\n");
	}
	else
	{
		log_debug( "Mail sent with ". $conf{'sendmail'} . "\n");
	}

#log_debug( $content);
}

sub contentsend_foreach($$$)
{
	my($file, $rev_old, $rev_new) = @_;
	my($email);
	$email = get_matched_value( $file, "attach_notify", "cvsnotify-contentsend" );
	if( $email )
	{
		do_attach_notify( $email, $file, $rev_old, $rev_new );
	}
	$email = get_matched_value( $file, "diff_notify", "cvsnotify-contentsend" );
	if( $email )
	{
		do_diff_notify( $email, $file, $rev_old, $rev_new );
	}
}

sub contentsend_postprocess()
{
	return 1;
}

# End of script.
1
